/*
 * Berkeley Lab Checkpoint/Restart (BLCR) for Linux is Copyright (c)
 * 2007, The Regents of the University of California, through Lawrence
 * Berkeley National Laboratory (subject to receipt of any required
 * approvals from the U.S. Dept. of Energy).  All rights reserved.
 *
 * Portions may be copyrighted by others, as may be noted in specific
 * copyright notices within specific files.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: save_aux.c,v 1.2 2008/02/14 02:29:43 phargrov Exp $
 *
 * Simple program to generate mmaps() as requred for testing --save-* options
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <errno.h>

int main(int argc, const char **argv) {
    volatile char **addr = calloc(argc, sizeof(char *));
    pid_t pid;
    int pagesize = getpagesize();
    sigset_t mask;
    int i;

    for (i=1; i <argc; ++i) {
	char filename[128];
	int flags, fd;

	/* Parse mode arg */
	switch (argv[i][0]) {
	case 'S': flags = MAP_SHARED; break;
	case 'P': flags = MAP_PRIVATE; break;
	default:  fprintf(stderr, "Invalid argument '%s'\n", argv[i]);
		  exit(-1);
	}

	/* Create a file */
	snprintf(filename, sizeof(filename), "mapped_%d", i);
	fd = open(filename, O_CREAT|O_RDWR|O_TRUNC, S_IREAD|S_IWRITE); 
	if (fd < 0) {
		perror("open()");
		exit(-1);
	}

	/* Extend to 2 pages in length */
	if (ftruncate(fd, 2*pagesize) < 0) {
		perror("ftruncate()");
		exit(-1);
	}

	/* mmap() it */
	addr[i] = mmap(NULL, 2*pagesize, PROT_READ|PROT_WRITE, flags, fd, 0);
	if (addr[i] == MAP_FAILED) {
		perror("mmap()");
		exit(-1);
	}
	(void)close(fd);

	/* dirty first page */
	*addr[i] = 1;
    }

    /* Fork a child to do like us */
    pid = fork();

    /* Wait upto 30 seconds for the checkpoint */
    (void)sigemptyset(&mask);
    (void)alarm(30);
    (void)sigsuspend(&mask);
    (void)alarm(0);

    /* Touch again */
    for (i=1; i <argc; ++i) {
	if (*addr[i] != 1) {
	    fprintf(stderr, "Failed to read back from first page\n");
	    return 1;
	}
	if (*(addr[i] + pagesize) != 0) {
	    fprintf(stderr, "Failed to read back from last page\n");
	    return 1;
	}
    }

    /* Parent waits for child */
    if (pid) {
	int rc, status;
	do { rc = waitpid(pid, &status, 0); } while ((rc < 0) && (errno == EINTR));
	if ((rc != pid) || !WIFEXITED(status) || (WEXITSTATUS(status) != 0)) {
	    return 1;
	}
    }

    return 0;
}
